﻿using System;
using System.IO;
using System.IO.Ports;
using System.Threading;
using GpsShield;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware.Netduino;
using SecretLabs.NETMF.IO;

namespace GpsDemo
{
    public class Program
    {
        private const string _root = "SD_Card";
		private static string _path;
		private static FileStream _fileStream;
		private static StreamWriter _writer;

		private static readonly OutputPort _led2 = new OutputPort(Pins.GPIO_PIN_D3, false);

        public static void Main()
        {
			// Mount the SD card
			//
            StorageDevice.MountSD(_root, SPI.SPI_module.SPI1, Cpu.Pin.GPIO_Pin12);

			// Determine the next file name.
			//
            int fileIndex = 0;
        	do
        	{
                fileIndex++;
				_path = Path.Combine(_root, "gpsData" + fileIndex + ".txt");
        		
        	} while (File.Exists(_path));

			// Open the file stream and the stream writer.
			//
            _fileStream = new FileStream(_path, FileMode.CreateNew, FileAccess.Write, FileShare.None, 512);
            _writer = new StreamWriter(_fileStream);

			// Open the serial port and turn the GPS unit on.

			Thread.Sleep(500);	// The GPS seems to need a little delay
			SerialPort serialPort = new SerialPort("COM1", 4800, Parity.None, 8, StopBits.One);
			OutputPort powerPin = new OutputPort(Pins.GPIO_PIN_D2, false);

			// Create a gps reader, hook up the event, and start the reader thread
			//
			Reader gpsShield = new Reader(serialPort, 100, 0.1);
			gpsShield.GpsData += GpsShield_GpsData;
			gpsShield.Start();

			// The main thread is done.
			//
        	Thread.Sleep(Timeout.Infinite);

			//DateTime start = DateTime.Now;
			//while (true)
			//{
			//    TimeSpan span = DateTime.Now - start;
			//    string line = span + ", count=" + _count;
			//    _writer.WriteLine(line);
			//    _writer.Flush();
			//    Debug.Print(line);
			//}
			//gpsShield.Stop();
			//_writer.Close();
			//StorageDevice.Unmount(_root);
			//Debug.Print("Main...exiting");
        }

        private static void GpsShield_GpsData(GpsPoint gpsPoint, string rawLine)
        {
			try
			{
				// Write the raw line to the data file.
				//
                _writer.WriteLine(rawLine);
				_writer.Flush();
                _fileStream.Flush();

				// Flash the led to indicate.
				//
            	_led2.Write(true);
            	Thread.Sleep(200);
            	_led2.Write(false);
			}
			catch (Exception ex)
			{
//				Debug.Print("Exception: " + ex);
			}
        }
    }
}
